#include "cellcalibration.h"

#include <QDebug>
#include <QVector>

#include "drs.h"

CellCalibration::CellCalibration()
{
    clear();
}

void CellCalibration::addCellAmpData(int channel, int startcell, const QVector<int> &data)
{
    if (data.size() > DRS_SIZE) { qDebug() << "AmpCalib error"; return; }
    for (int i=0; i<data.size(); i++)
    {
        int realcell = (i + startcell) % DRS_SIZE;
        cellAmpSum[channel][realcell] += data[i];
        cellAmpEntries[channel][realcell]++;
    }
    calcCellAmpCalibration(channel);
    if(channel == 0) qDebug() << "startcell=" << startcell;
}

void CellCalibration::calcCellAmpCalibration(int channel)
{
    for (int i=0; i<DRS_SIZE; i++)
    {
        int numEntries = cellAmpEntries[channel][i];
        if (numEntries > 0)
            cellAmpCalibration[channel][i] = 1.0*cellAmpSum[channel][i] / numEntries;
        else
            cellAmpCalibration[channel][i] = 0;
    }
}

QVector<double> CellCalibration::getCellAmpCalibration(int channel) const
{
    return cellAmpCalibration[channel];
}

void CellCalibration::addCellAmpCalibration(int channel, QVector<double> calib)
{
    for(int i=0; i<calib.size(); i++)
        cellAmpCalibration[channel][i] = calib[i];
}

void CellCalibration::clear()
{
    for(int i=0; i<DRS_CHANNELS; i++) {
        cellAmpCalibration[i] = QVector<double>(DRS_SIZE, 0.);
        cellAmpSum[i] = QVector<double>(DRS_SIZE, 0.);
        cellAmpEntries[i] =  QVector<int>(DRS_SIZE, 0);
    }
}
