#ifndef DOMINODEVICE_H
#define DOMINODEVICE_H

#include <QFile>
#include <QTextStream>
#include <QTimer>
#include <QMap>
#include <QDateTime>

#include "dominodata.h"
#include "cellcalibration.h"
#include "dominosettings.h"
#include <regio/qtmregdevice.h>

namespace mlink {
class RegOpVector;
}

class DominoDevice : public QtMregDevice
{
    Q_OBJECT
public:
    DominoDevice(QObject *parent = 0);
    void writeSettings(const DominoSettings &settings, bool init);
    void StopRun();
    bool isRunning();
    DominoData ReadAllChMem();
    DominoData applyAmpCalibration(const DominoData &dominoData) const;
    void CalibrateCellAmp(const DominoData &dominoData);
    void SaveData(const DominoData &dominoData);
    void StartRun();
//    DominoRealData getDominoRealData() const {return dominoData.Data;}

//    void clearEventNumber() { dominoData.EventNumber=0; }
    void clearCellCalib() { cellCalibration.clear(); }
    bool RegWRTest(int NCycle);
    void writeCellCalibToFile();
    void readCellCalibFromFile();
    bool ActiveRun;

    QString DataFileName;
    QFile DataFile;
    QTextStream DataOut;

private:

    mlink::RegOpVector DacWrite(int dacnum, int ch, int data);
    mlink::RegOpVector SetMode();
    mlink::RegOpVector deviceInit();
    void ad9252_spi_write(quint16 addr, quint8 data);
    quint16 ad9252_spi_read(quint16 addr);
    void ad9252_setup();

    CellCalibration cellCalibration;
    DominoSettings setup;
    QDateTime comp_time;

//    void SetCntMode(quint16 mask);
    DominoChannelData ReadChMem(int ch);

private Q_SLOTS:
//    void setSetup(const DominoSettings &);

Q_SIGNALS:
};

#endif // DOMINODEVICE_H
