//
//    Copyright 2010 Alexey Shutov, Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "dominosettings.h"
#include "drs.h"

DominoSettings::DominoSettings()
    : mode(NORMAL_MODE) , readcellnumber(DRS_SIZE) ,
      writecellnumber(DRS_SIZE) , trigdelay(4), analogDelay(1600)
{
    resize(DRS_CHANNELS);
}

void DominoSettings::resize(int n)
{
    en.clear();
    thr.clear();
    trig.clear();
    for(int i=0; i<n; i++) {
        en[i] = true;
        thr[i] = 1950;
        trig[i] = true;
    }
}

bool DominoSettings::operator==(const DominoSettings &other) const
{
    return true
            && en == other.en
            && thr == other.thr
            && trig == other.trig
            && mode == other.mode
            && readcellnumber == other.readcellnumber
            && writecellnumber == other.writecellnumber
            && trigdelay == other.trigdelay
            && analogDelay == other.analogDelay
            ;
}

quint32 DominoSettings::getEnMask() const
{
    quint32 result = 0;
    for(int i=0; i<DRS_CHANNELS; i++) {
        if (en[i])
            result |= (0x1<<i);
    }
    return result;
}

quint32 DominoSettings::getTrigMask() const
{
    quint32 result = 0;
    for(int i=0; i<DRS_CHANNELS; i++) {
        if (trig[i])
            result |= (0x1<<i);
    }
    return result;
}
