#ifndef MAINWINDOW_H
#define MAINWINDOW_H


#include <QMainWindow>
#include <QAbstractButton>
#include <QDateTime>

#include "qxw/WaveformData.h"
#include "dominosettings.h"

namespace Ui {
    class MainWindow;
}

class QTimer;
class QLabel;
class DominoDevice;
class DominoData;

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();


Q_SIGNALS:
    void gotData(const DominoData &);


protected:
    void changeEvent(QEvent *e);

private:
    void readSettings();
    void writeSettings();
    void resetSettings();
    void readCellAmpCalib();
    virtual void closeEvent(QCloseEvent *event);
    void setBoardHostName();
    void restoreFormValues();
    DominoData applyAmpCalibration(const DominoData &dominoData) const;

    Ui::MainWindow * const ui;
    QLabel *statusLabel;
    QTimer *tmr;
    DominoDevice * const dominoDevice;
    DominoSettings setup;
    QString lastSuccessfulHostAddress;
    QDateTime scopeUpdateTime;

private Q_SLOTS:
    void DrawScope(const DominoData &dominoRealData);
    void saveData(const DominoData &dominoData);
    void appendCalibrationData(const DominoData &dominoData);
    void onDeviceConnect();
    void onDeviceDisconnect();
    void setStatus(const QString &s);
    void setTemp(double);
    void on_buttonBoxHostName_clicked(QAbstractButton* button);
    void on_spinBoxAfterTrig_valueChanged(int );
    void on_checkBoxTr8_toggled(bool checked);
    void on_checkBoxTr7_toggled(bool checked);
    void on_checkBoxTr6_toggled(bool checked);
    void on_checkBoxTr5_toggled(bool checked);
    void on_checkBoxTr4_toggled(bool checked);
    void on_checkBoxTr3_toggled(bool checked);
    void on_checkBoxTr2_toggled(bool checked);
    void on_checkBoxTr1_toggled(bool checked);
    void on_checkBoxE8_toggled(bool checked);
    void on_checkBoxE7_toggled(bool checked);
    void on_checkBoxE6_toggled(bool checked);
    void on_checkBoxE5_toggled(bool checked);
    void on_checkBoxE4_toggled(bool checked);
    void on_checkBoxE3_toggled(bool checked);
    void on_radioButtonNormMode_toggled(bool checked);
    void on_radioButtonAOffMod_toggled(bool checked);
    void on_spinBoxT8_valueChanged(int );
    void on_spinBoxT7_valueChanged(int );
    void on_spinBoxT6_valueChanged(int );
    void on_spinBoxT5_valueChanged(int );
    void on_spinBoxT4_valueChanged(int );
    void on_spinBoxT3_valueChanged(int );
    void on_spinBoxT2_valueChanged(int );
    void on_checkBoxW2D_toggled(bool checked);
    void on_pushButtonTest2_clicked();
    void on_checkBoxE2_toggled(bool checked);
    void on_pushButton_setdac_clicked();
    void on_pushButton_rm_clicked();
    void on_pushButton_t2_clicked();
    void on_pushButton_t1_clicked();
    void on_checkBoxE1_toggled(bool checked);
    void on_spinBoxADelay_valueChanged(int );
    void on_spinBoxWriteCNT_valueChanged(int );
    void on_spinBoxReadCNT_valueChanged(int );
    void on_spinBoxT1_valueChanged(int );
    void on_pushButtonStop_clicked();
    void on_pushButtonStart_clicked();
    void DoTimer();
    void DoStopTimer();
    void on_actionQuit_activated();
    void on_actionDocs_activated();
    void on_actionCheckForUpdates_activated();
    void on_actionAbout_activated();
    void on_actionAboutQt_activated();
    void on_pushButtonResetSettings_clicked();
};

#endif // MAINWINDOW_H
