//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef MREGDEVICE_H
#define MREGDEVICE_H

#include "regio/mlinkdevice.h"

namespace mlink {

class DeviceAttr {
public:
    DeviceAttr();
    QString getFirmwareVersionStr() const;

    quint16 device_id;
    bool id_chip_present;
    quint64 device_serial;
    double temp;
    double clock;
    quint32 fw_ver;
    quint32 fw_rev;
};

class MregDevice : public MlinkDevice
{
public:
    enum {
        REG_DEVICE_ID = 0x42
    };
    MregDevice(RegIOBase &p, int address);
    virtual ~MregDevice() {}
    virtual void print() const;
    virtual void detect();
    DeviceAttr getAttr() const { return attr; }

    virtual double readTemp() = 0;
    virtual quint64 readTimestampCurrent() = 0;
    virtual quint64 readTimestampRun() = 0;
    virtual quint64 readTimestampTrig() = 0;
    quint16 readDeviceId();

protected:
    quint16 getDeviceId() const { return attr.device_id; }
    quint64 getDeviceSerial() const { return attr.device_serial; }
    virtual quint64 readOneWireId() = 0;
    virtual bool readFirmwareId() = 0;

    DeviceAttr attr;

};

class MregDeviceDummy : public MregDevice
{
public:
    MregDeviceDummy(RegIOBase &link, int address): MregDevice(link, address) {}
    virtual ~MregDeviceDummy() {}
    virtual double readTemp() {return 0;}
    virtual quint64 readTimestampCurrent() {return 0;}
    virtual quint64 readTimestampRun() {return 0;}
    virtual quint64 readTimestampTrig() {return 0;}

protected:
//    virtual quint16 readDeviceId();
    virtual quint64 readOneWireId() {return 0;}
    virtual bool readFirmwareId() {return false;}
};

class MregDeviceBpm : public MregDevice
{
public:
    enum {
        REG_DEVICE_ID = 0x42, //
        REG64_SERIAL_NUM = 0x50, //

        REG_TEMPERATURE = 0x54, // 84

        REG_FW_VER = 0x56, // 86
        REG_FW_REV = 0x57, // 87

        REG_LINK_ERR_CNT_1 = 0x58,
        REG_LINK_ERR_CNT_2 = 0x59,
        REG_LINK_ERR_CNT_3 = 0x5A,
        REG_LINK_CRC_CNT_1 = 0x5B,
        REG_LINK_CRC_CNT_2 = 0x5C,
        REG_LINK_CRC_CNT_3 = 0x5D,

        REG_LINK_STATUS = 0x61,

        REG64_CNT_NCU = 0x70,
        REG64_CNT_KCU = 0x74,
        REG64_CURRENT_TIMESTAMP = 0x78,
    };

    MregDeviceBpm(RegIOBase &p, int address);
    virtual ~MregDeviceBpm() {}
    virtual double readTemp();
    virtual quint64 readTimestampCurrent();
    virtual quint64 readTimestampRun();
    virtual quint64 readTimestampTrig();

protected:
//    virtual quint16 readDeviceId();
    virtual quint64 readOneWireId();
    virtual bool readFirmwareId();
};

class MregDeviceAdcm : public MregDevice
{
public:
    enum {
        // Control registers
        ADCM16_CTRL_REG           = 0x40,
        ADCM16_CTRL_DEFAULT     = 0x0000,

        ADCM16_WIN_REG            = 0x41,
        ADCM16_STATUS_REG         = 0x42, /* read-only */
        ADCM16_WIN_DEFAULT      = 0xFFFF,

        ADCM16_TRIGMULT_REG       = 0x43,
        ADCM16_FRAGMLEN_REG       = 0x44,
        ADCM16_FRAGMMAX_REG       = 0x45, /* read-only */
        ADCM16_FIFO_PFULL_REG     = 0x46,
        ADCM16_MAX_NFRAG_REG      = 0x47, /* read-only */


        ADCM16_EXT_LAT_REG  = ADCM16_WIN_REG,
        ADCM16_EXT_MATCHWIN_REG   = 0x48,
        ADCM16_EXT_PREWIN_REG     = 0x49,
        ADCM16_CH_DPM_KS_REG      = 0x4A, /* read-only */
        ADCM16_TEMPERATURE_REG    = 0x4B, /* read-only */
        ADCM16_FW_VER_REG         = 0x4C, /* read-only */
        ADCM16_FW_REV_REG         = 0x4D, /* read-only */
        ADCM16_SERIAL_ID_REG      = 0x4E, /* read-only */
        ADCM16_PROPERTIES_REG     = 0x4F,  /* read-only */

        // Timestamp registers
        ADCM16_TS_READ_A          = 0x50,
        ADCM16_TS_SET_REG64       = 0x54,
        ADCM16_TS_READ_B          = 0x58,
        ADCM16_TS_READ_REG64      = 0x5C,
    };
/*
    enum {
        REG_DEVICE_ID = 0x42, //
        REG64_SERIAL_NUM = 0x50, //

        REG_TEMPERATURE = 0x54, // 84

        REG_FW_VER = 0x56, // 86
        REG_FW_REV = 0x57, // 87

        REG_LINK_ERR_CNT_1 = 0x58,
        REG_LINK_ERR_CNT_2 = 0x59,
        REG_LINK_ERR_CNT_3 = 0x5A,
        REG_LINK_CRC_CNT_1 = 0x5B,
        REG_LINK_CRC_CNT_2 = 0x5C,
        REG_LINK_CRC_CNT_3 = 0x5D,

        REG_LINK_STATUS = 0x61,
    };
*/
    MregDeviceAdcm(RegIOBase &p, int address);
    virtual ~MregDeviceAdcm() {}
    virtual double readTemp();
    virtual quint64 readTimestampCurrent();
    virtual quint64 readTimestampRun();
    virtual quint64 readTimestampTrig();

protected:
//    virtual quint16 readDeviceId();
    virtual quint64 readOneWireId();
    virtual bool readFirmwareId();
};

} // namespace mlink

#endif // MREGDEVICE_H
