
#include "regiobase.h"

#include <QDebug>
#include <stdexcept>
#include <sstream>
#include <iostream>
#include <algorithm>

#include <QCoreApplication>

#include "mlink/frame_types.h"
#include "mlink/mlink_frame_header.h"
#include "util/gettimeofday.h"
#include "frame_util.h"
#include "TextFormatter.h"

namespace mlink {

double getCurrentTimestamp()
{
    struct timeval tv;
    gettimeofday(&tv, NULL);
    return tv.tv_sec + 1e-6*tv.tv_usec;
}

RegIOBase::RegIOBase(QObject *parent, const QHostAddress &address)
    : QObject(parent), deviceAddress(address), bufSizeLimit(0)
{
}

RegIOBase::~RegIOBase()
{
}

std::string RegIOBase::getDevName() const
{
    return "mlink-netdev";
}

unsigned long RegIOBase::getDriverVersion() const
{
    return 0;
}

unsigned long RegIOBase::getDriverRevision() const
{
    return 0;
}

void RegIOBase::setHostAddress(const QHostAddress &address)
{
    deviceAddress = address;
}

const double protocolTimeout = 1.0;

bool RegIOBase::packetTypeIsData(int type)
{
    if (type == FRAME_TYPE_ADC_DATA) return true;
    if (type == FRAME_TYPE_STAT) return true;
    return false;
}

void RegIOBase::sendPackets(const PacketList &pl)
{
    for(PacketList::const_iterator it = pl.begin(); it != pl.end(); ++it) {
        rawSend(*it);
    }
}

void RegIOBase::reset()
{
    rxBufCtrl.clear();
    //    rxBufData.clear();
}

double RegIOBase::getDeviceThroughput() const
{
    return 1e6; // default value
}

size_t RegIOBase::getCtrlPacketCount() const
{
    return rxBufCtrl.size();
}

PacketList RegIOBase::getCtrlPacketBySeq(int seq, double timeout)
{
    bool found = false;
    double tv1 = getCurrentTimestamp();
    while(true)
    {
        //        receive();
//        qDebug() << rxBufCtrl.size();
        for(PacketList::const_iterator it = rxBufCtrl.begin(); it != rxBufCtrl.end(); ++it) {
//            qDebug() << "seq = " << it->seq << ", expected " << seq;
            if (it->seq == seq) {
//                qDebug() << "Ok: rxBufCtrl=" << rxBufCtrl;
                found = true;
                break;
            }
        }
        if (found) break;
        double tv2 = getCurrentTimestamp();
        double dt = tv2 - tv1;
        if (dt > timeout) {
//            qWarning() << "Protocol timeout, dt=" << dt << " s (getCtrlPacketBySeq(" << seq << ", " << timeout << "))";
//            qDebug() << "rxBufCtrl=" << rxBufCtrl;
            break;
        }
        qApp->processEvents();
    }
    const PacketList pl(rxBufCtrl);
    rxBufCtrl.clear();
    return pl;
}

PacketList RegIOBase::getCtrlPackets(size_t packetCount, double timeout)
{
    double tv1 = getCurrentTimestamp();
    while(getCtrlPacketCount() < packetCount)
    {
        //        receive();
        if (getCtrlPacketCount() >= packetCount) break;
        double tv2 = getCurrentTimestamp();
        double dt = tv2 - tv1;
        if (dt > timeout) {
            qWarning() << "Protocol timeout, dt=" << dt << " s (getControlPackets(" << packetCount << ", " << timeout << "))";
            qDebug() << "rxBufCtrl=" << rxBufCtrl;
            break;
        }
        qApp->processEvents();
    }
    const PacketList pl(rxBufCtrl);
    rxBufCtrl.clear();
    return pl;
}

} // namespace mlink
