#ifndef REGIO_REGIOBASE_H
#define REGIO_REGIOBASE_H

#include <cstddef>
#include <string>

#include <QtGlobal>
#include <QObject>
#include <QHostAddress>

#include <regio/regiopacket.h>

#if ((__GNUC__ > 4) || (__GNUC__ == 4 && __GNUC_MINOR__ >= 1))
#define WARN_UNUSED  __attribute__ ((warn_unused_result))
#else
#define WARN_UNUSED
#endif

namespace mlink {

class RegIOBase: public QObject
{
    Q_OBJECT

public:
    RegIOBase(QObject *parent, const QHostAddress &address = QHostAddress()); ///< Construct M-Link Register I/O object with specified device address
    virtual ~RegIOBase();
    void setHostAddress(const QHostAddress &address);
    QHostAddress getHostAddress() const { return deviceAddress; }
    virtual void reset();
    virtual void rawSend(const RegIoPacket &data) = 0; ///< \deprecated use sendPackets()
    virtual void sendPackets(const PacketList &);
    virtual size_t getCtrlPacketCount() const; ///< return number of control packets received
    PacketList getCtrlPacketBySeq(int seq, double timeout);
    virtual PacketList getCtrlPackets(size_t packetCount, double timeout) WARN_UNUSED;
    virtual unsigned long getMaxDataSize() const = 0;

    std::string getDevName() const;
    unsigned long getDriverVersion() const;
    unsigned long getDriverRevision() const;

Q_SIGNALS:
    void rawDataReceived(const QByteArray &);

protected:
    virtual double getDeviceThroughput() const;
    bool packetTypeIsData(int type);

    QHostAddress deviceAddress;

    int bufSizeLimit;
    PacketList rxBufCtrl;
};

double getCurrentTimestamp();

} // namespace mlink

#endif /* REGIO_REGIOBASE_H */
