#ifndef CANVASPICKER_H
#define CANVASPICKER_H

#include <QObject>
#include <QUndoStack>
#include <QUndoCommand>

#include <qxw/qxw_global.h>

class QPoint;
class QCustomEvent;
class QwtPlot;
class QwtPlotCurve;

class QXW_EXPORT CurveSelection
{
public:
    enum { Point, Segment };
    CurveSelection();
    CurveSelection(QwtPlotCurve *, double dist, double index, int type);
    void clear();
    bool empty() const { return c==0 || index < 0; }
    bool isPoint() const { return !empty() && type == Point; }
    bool isSegment() const { return !empty() && type == Segment; }

    QwtPlotCurve *c;
    double dist;
    int index;
    int type;
};

class QwtPlotEdit;

class QXW_EXPORT MoveCommand: public QUndoCommand
{
public:
    MoveCommand(QwtPlotEdit *picker, QwtPlotCurve *c, int index, double dx, double dy);
    virtual void undo();
    virtual void redo();
    virtual bool mergeWith(const QUndoCommand *command);
    virtual int id() const { return 12345; }
    QString myText() const;
private:
    void doMove(bool forward = true);
    QwtPlotEdit *myPicker;
    QwtPlotCurve *myCurve;
    int myIndex;
    double myDx, myDy;
};

class QXW_EXPORT AddCommand: public QUndoCommand
{
public:
    AddCommand(QwtPlotEdit *picker, QwtPlotCurve *c, int index, double px, double py);
    virtual void undo();
    virtual void redo();
    virtual int id() const { return 12345+1; }
    QString myText() const;
protected:
    enum {ActionAdd, ActionRemove };
    void doAddRemove(int action);
    void doAdd() { doAddRemove(ActionAdd); }
    void doRemove() { doAddRemove(ActionRemove); }
    QwtPlotEdit *myPicker;
    QwtPlotCurve *myCurve;
    int myIndex;
    double myPx, myPy;
};

class QXW_EXPORT RemoveCommand: public AddCommand
{
public:
    RemoveCommand(QwtPlotEdit *picker, QwtPlotCurve *c, int index);
    virtual void undo();
    virtual void redo();
    virtual int id() const { return 12345+2; }
    QString myText() const;
protected:
};

class QXW_EXPORT QwtPlotEdit: public QObject
{
    Q_OBJECT
public:
    QwtPlotEdit(QwtPlot *plot, QUndoStack *undoStack);
    virtual bool eventFilter(QObject *, QEvent *);

    virtual bool event(QEvent *);
    void setMouseButton(int button, int buttonState = Qt::NoButton);
    void updateSelection();

public slots:
    void redo();
    void undo();

private:
    void select(const QPoint &);
    void move(const QPoint &);
    void moveBy(int dx, int dy);

    void release();

    void addPoint(QwtPlotCurve *c, const QPoint &p, int index = -1);
    void deletePoint();
    void showCursor(bool showIt);
    void newSelection(CurveSelection select);
    void shiftPointCursor(bool up);
    void shiftCurveCursor(bool up);

    QwtPlot *plot() { return reinterpret_cast<QwtPlot *>(parent()); }
    const QwtPlot *plot() const { return reinterpret_cast<const QwtPlot *>(parent()); }

    CurveSelection d_select;
    int pickerButton;
    QUndoStack *undoStack;
};

#endif // CANVASPICKER_H
